//
//  TEMDocument.swift
//  TextEditMM
//
//  Created by narita on 2019/10/24.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

class TEMDocument: NSDocument
{
    // MARK: - class methods
    
    override class var autosavesInPlace: Bool
    {
        return true
    }
    
    // MARK: - init methods
    
    override init()
    {
        super.init()
        // Add your subclass-specific initialization here.
        
        self.textStorage =  NSTextStorage()
    }

    public convenience init(contentsOf url: URL, ofType typeName: String) throws
    {
        self.init()
        
        try self.read(from: url, ofType: typeName)
        
        self.fileURL = url
        self.fileType = typeName
        
        let theAttr = try FileManager.default.attributesOfFileSystem(forPath: url.path)
        
        self.fileModificationDate = theAttr[FileAttributeKey.modificationDate] as? Date
    }

    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods

    override func fileWrapper(ofType typeName: String) throws -> FileWrapper
    {
        // self.textStorageがnilならばエラー
        if self.textStorage == nil
        {
            NSException(
                name: NSExceptionName.objectNotAvailableException,
                reason: "self.textStorage is nil.").raise()
        }

        
        let theWorkspace = NSWorkspace.shared
        let theRange: NSRange  = NSMakeRange(0, self.textStorage!.length)
        
        if theWorkspace.type(kUTTypePlainText as String, conformsToType: typeName)
        {
            return
                try self.textStorage!.fileWrapper(from: theRange,
                                                  documentAttributes:
                                [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.plain])

        }
        else
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }
        
        return FileWrapper()
    }
    
    override func read(from fileWrapper: FileWrapper, ofType typeName: String) throws
    {
        let theWorkspace = NSWorkspace.shared
        
        if theWorkspace.type(kUTTypePlainText as String, conformsToType: typeName)
        {
            self.textStorage = try NSTextStorage(data: fileWrapper.regularFileContents!,
                                                 options: [:],
                                                 documentAttributes: nil)
        }
        else
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }
    }

    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var textStorage: NSTextStorage?
    
    // MARK: - Utility methods
    
    override func makeWindowControllers()
    {
        // Returns the Storyboard that contains your Document window.
        let storyboard = NSStoryboard(name: "TEMWindowController", bundle: nil)
        
        let windowController = storyboard.instantiateInitialController() as! TEMWindowController
        
        self.addWindowController(windowController)
    }

}

